import chalk from 'chalk';
import createExpress from 'express';
import * as HTTP from 'http';
import { isEmpty } from 'lodash';
import { httpClose, httpListen, noop } from '../../lib/utils';
import * as DevMode from '../dev-mode';
import { assembledGuard } from '../utils';
import { ApolloServerExpress } from './apollo-server';
import { errorFormatter } from './error-formatter';
import { createRequestHandlerGraphQL } from './handler-graphql';
import { log } from './logger';
import { createServerSettingsManager } from './settings';
const resolverLogger = log.child('apollo');
export const defaultState = {
    running: false,
    httpServer: HTTP.createServer(),
    createContext: null,
    apolloServer: null,
    enableSubscriptionsServer: false,
};
export function create(appState) {
    const settings = createServerSettingsManager();
    const express = createExpress();
    const state = Object.assign({}, defaultState);
    const api = {
        raw: {
            http: state.httpServer,
        },
        express,
        handlers: {
            get graphql() {
                var _a;
                return ((_a = assembledGuard(appState, 'app.server.handlers.graphql', () => {
                    if (settings.data.cors.enabled) {
                        log.warn('CORS does not work for serverless handlers. Settings will be ignored.');
                    }
                    return createRequestHandlerGraphQL(appState.assembled.schema, appState.assembled.createContext, {
                        path: settings.data.path,
                        introspection: settings.data.graphql.introspection,
                        playground: settings.data.playground.enabled ? settings.data.playground : false,
                        errorFormatterFn: errorFormatter,
                    });
                })) !== null && _a !== void 0 ? _a : noop);
            },
        },
    };
    const internalServer = {
        private: {
            settings,
            state,
            reset() {
                internalServer.private.state = Object.assign({}, defaultState);
            },
            assemble(loadedRuntimePlugins, schema) {
                state.httpServer.on('request', express);
                const createContext = createContextCreator(appState.components.schema.contextContributors, loadedRuntimePlugins);
                /**
                 * Resolve if subscriptions are enabled or not
                 */
                if (settings.metadata.fields.subscriptions.fields.enabled.from === 'change') {
                    state.enableSubscriptionsServer = settings.data.subscriptions.enabled;
                    /**
                     * Validate the integration of server subscription settings and the schema subscription type definitions.
                     */
                    if (hasSubscriptionFields(schema)) {
                        if (!settings.data.subscriptions.enabled) {
                            log.error(`You have disabled server subscriptions but your schema has a ${chalk.yellowBright('Subscription')} type with fields present. When your API clients send subscription operations at runtime they will fail.`);
                        }
                    }
                    else if (settings.data.subscriptions.enabled) {
                        log.warn(`You have enabled server subscriptions but your schema has no ${chalk.yellowBright('Subscription')} type with fields.`);
                    }
                }
                else if (hasSubscriptionFields(schema)) {
                    state.enableSubscriptionsServer = true;
                }
                /**
                 * Setup Apollo Server
                 */
                state.apolloServer = new ApolloServerExpress({
                    schema,
                    engine: settings.data.apollo.engine.enabled ? settings.data.apollo.engine : false,
                    // todo expose options
                    subscriptions: settings.data.subscriptions,
                    context: createContext,
                    introspection: settings.data.graphql.introspection,
                    formatError: errorFormatter,
                    logger: resolverLogger,
                    playground: settings.data.playground.enabled
                        ? {
                            endpoint: settings.data.path,
                            settings: settings.data.playground.settings,
                        }
                        : false,
                });
                state.apolloServer.applyMiddleware({
                    app: express,
                    path: settings.data.path,
                    cors: settings.data.cors,
                });
                if (state.enableSubscriptionsServer) {
                    state.apolloServer.installSubscriptionHandlers(state.httpServer);
                }
                return { createContext };
            },
            async start() {
                await httpListen(state.httpServer, { port: settings.data.port, host: settings.data.host });
                state.running = true;
                // About !
                // 1. We do not support listening on unix domain sockets so string
                //    value will never be present here.
                // 2. We are working within the listen callback so address will not be null
                const address = state.httpServer.address();
                settings.data.startMessage({
                    port: address.port,
                    host: address.address,
                    ip: address.address,
                    paths: {
                        graphql: settings.data.path,
                        graphqlSubscriptions: state.enableSubscriptionsServer ? settings.data.subscriptions.path : null,
                    },
                });
                DevMode.sendServerReadySignalToDevModeMaster();
            },
            async stop() {
                var _a;
                if (!state.running) {
                    log.warn('You called `server.stop` but the server was not running.');
                    return Promise.resolve();
                }
                await httpClose(state.httpServer);
                await ((_a = state.apolloServer) === null || _a === void 0 ? void 0 : _a.stop());
                state.running = false;
            },
        },
        public: api,
    };
    return internalServer;
}
/**
 * Combine all the context contributions defined in the app and in plugins.
 */
function createContextCreator(contextContributors, plugins) {
    const createContext = async (params) => {
        let context = {};
        // Integrate context from plugins
        for (const plugin of plugins) {
            if (!plugin.context)
                continue;
            const contextContribution = await plugin.context.create(params.req);
            Object.assign(context, contextContribution);
        }
        // Integrate context from app context api
        // TODO good runtime feedback to user if something goes wrong
        for (const contextContributor of contextContributors) {
            const contextContribution = await contextContributor(params);
            Object.assign(context, contextContribution);
        }
        Object.assign(context, { log: log.child('request') });
        return context;
    };
    return createContext;
}
function hasSubscriptionFields(schema) {
    var _a;
    return !isEmpty((_a = schema.getSubscriptionType()) === null || _a === void 0 ? void 0 : _a.getFields());
}
//# sourceMappingURL=server.js.map