"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.create = exports.defaultState = void 0;
const tslib_1 = require("tslib");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const express_1 = tslib_1.__importDefault(require("express"));
const HTTP = tslib_1.__importStar(require("http"));
const lodash_1 = require("lodash");
const utils_1 = require("../../lib/utils");
const DevMode = tslib_1.__importStar(require("../dev-mode"));
const utils_2 = require("../utils");
const apollo_server_1 = require("./apollo-server");
const error_formatter_1 = require("./error-formatter");
const handler_graphql_1 = require("./handler-graphql");
const logger_1 = require("./logger");
const settings_1 = require("./settings");
const resolverLogger = logger_1.log.child('apollo');
exports.defaultState = {
    running: false,
    httpServer: HTTP.createServer(),
    createContext: null,
    apolloServer: null,
    enableSubscriptionsServer: false,
};
function create(appState) {
    const settings = settings_1.createServerSettingsManager();
    const express = express_1.default();
    const state = Object.assign({}, exports.defaultState);
    const api = {
        raw: {
            http: state.httpServer,
        },
        express,
        handlers: {
            get graphql() {
                var _a;
                return ((_a = utils_2.assembledGuard(appState, 'app.server.handlers.graphql', () => {
                    if (settings.data.cors.enabled) {
                        logger_1.log.warn('CORS does not work for serverless handlers. Settings will be ignored.');
                    }
                    return handler_graphql_1.createRequestHandlerGraphQL(appState.assembled.schema, appState.assembled.createContext, {
                        path: settings.data.path,
                        introspection: settings.data.graphql.introspection,
                        playground: settings.data.playground.enabled ? settings.data.playground : false,
                        errorFormatterFn: error_formatter_1.errorFormatter,
                    });
                })) !== null && _a !== void 0 ? _a : utils_1.noop);
            },
        },
    };
    const internalServer = {
        private: {
            settings,
            state,
            reset() {
                internalServer.private.state = Object.assign({}, exports.defaultState);
            },
            assemble(loadedRuntimePlugins, schema) {
                state.httpServer.on('request', express);
                const createContext = createContextCreator(appState.components.schema.contextContributors, loadedRuntimePlugins);
                /**
                 * Resolve if subscriptions are enabled or not
                 */
                if (settings.metadata.fields.subscriptions.fields.enabled.from === 'change') {
                    state.enableSubscriptionsServer = settings.data.subscriptions.enabled;
                    /**
                     * Validate the integration of server subscription settings and the schema subscription type definitions.
                     */
                    if (hasSubscriptionFields(schema)) {
                        if (!settings.data.subscriptions.enabled) {
                            logger_1.log.error(`You have disabled server subscriptions but your schema has a ${chalk_1.default.yellowBright('Subscription')} type with fields present. When your API clients send subscription operations at runtime they will fail.`);
                        }
                    }
                    else if (settings.data.subscriptions.enabled) {
                        logger_1.log.warn(`You have enabled server subscriptions but your schema has no ${chalk_1.default.yellowBright('Subscription')} type with fields.`);
                    }
                }
                else if (hasSubscriptionFields(schema)) {
                    state.enableSubscriptionsServer = true;
                }
                /**
                 * Setup Apollo Server
                 */
                state.apolloServer = new apollo_server_1.ApolloServerExpress({
                    schema,
                    engine: settings.data.apollo.engine.enabled ? settings.data.apollo.engine : false,
                    // todo expose options
                    subscriptions: settings.data.subscriptions,
                    context: createContext,
                    introspection: settings.data.graphql.introspection,
                    formatError: error_formatter_1.errorFormatter,
                    logger: resolverLogger,
                    playground: settings.data.playground.enabled
                        ? {
                            endpoint: settings.data.path,
                            settings: settings.data.playground.settings,
                        }
                        : false,
                });
                state.apolloServer.applyMiddleware({
                    app: express,
                    path: settings.data.path,
                    cors: settings.data.cors,
                });
                if (state.enableSubscriptionsServer) {
                    state.apolloServer.installSubscriptionHandlers(state.httpServer);
                }
                return { createContext };
            },
            async start() {
                await utils_1.httpListen(state.httpServer, { port: settings.data.port, host: settings.data.host });
                state.running = true;
                // About !
                // 1. We do not support listening on unix domain sockets so string
                //    value will never be present here.
                // 2. We are working within the listen callback so address will not be null
                const address = state.httpServer.address();
                settings.data.startMessage({
                    port: address.port,
                    host: address.address,
                    ip: address.address,
                    paths: {
                        graphql: settings.data.path,
                        graphqlSubscriptions: state.enableSubscriptionsServer ? settings.data.subscriptions.path : null,
                    },
                });
                DevMode.sendServerReadySignalToDevModeMaster();
            },
            async stop() {
                var _a;
                if (!state.running) {
                    logger_1.log.warn('You called `server.stop` but the server was not running.');
                    return Promise.resolve();
                }
                await utils_1.httpClose(state.httpServer);
                await ((_a = state.apolloServer) === null || _a === void 0 ? void 0 : _a.stop());
                state.running = false;
            },
        },
        public: api,
    };
    return internalServer;
}
exports.create = create;
/**
 * Combine all the context contributions defined in the app and in plugins.
 */
function createContextCreator(contextContributors, plugins) {
    const createContext = async (params) => {
        let context = {};
        // Integrate context from plugins
        for (const plugin of plugins) {
            if (!plugin.context)
                continue;
            const contextContribution = await plugin.context.create(params.req);
            Object.assign(context, contextContribution);
        }
        // Integrate context from app context api
        // TODO good runtime feedback to user if something goes wrong
        for (const contextContributor of contextContributors) {
            const contextContribution = await contextContributor(params);
            Object.assign(context, contextContribution);
        }
        Object.assign(context, { log: logger_1.log.child('request') });
        return context;
    };
    return createContext;
}
function hasSubscriptionFields(schema) {
    var _a;
    return !lodash_1.isEmpty((_a = schema.getSubscriptionType()) === null || _a === void 0 ? void 0 : _a.getFields());
}
//# sourceMappingURL=server.js.map