"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createServerSettingsManager = void 0;
const tslib_1 = require("tslib");
const Setset = tslib_1.__importStar(require("setset"));
const Utils = tslib_1.__importStar(require("../../lib/utils"));
const logger_1 = require("./logger");
exports.createServerSettingsManager = () => Setset.create({
    fields: {
        subscriptions: {
            shorthand(enabled) {
                return { enabled };
            },
            fields: {
                path: {
                    initial() {
                        return '/graphql';
                    },
                },
                keepAlive: {},
                onConnect: {},
                onDisconnect: {},
                enabled: {
                    initial() {
                        // This is not accurate. The default is actually dynamic depending
                        // on if the user has defined any subscription type or not.
                        return true;
                    },
                },
            },
        },
        apollo: {
            fields: {
                engine: {
                    shorthand(enabled) {
                        return { enabled };
                    },
                    fields: {
                        enabled: {
                            initial() {
                                return false;
                            },
                        },
                    },
                },
            },
        },
        playground: {
            shorthand(enabled) {
                return { enabled };
            },
            fields: {
                enabled: {
                    initial() {
                        return process.env.NODE_ENV === 'production' ? false : true;
                    },
                },
                settings: {
                    fields: {
                        'editor.cursorShape': {
                            initial() {
                                return 'block';
                            },
                        },
                        'editor.fontFamily': {
                            initial() {
                                return `'Source Code Pro', 'Consolas', 'Inconsolata', 'Droid Sans Mono', 'Monaco', monospace`;
                            },
                        },
                        'editor.fontSize': {
                            initial() {
                                return 14;
                            },
                        },
                        'editor.reuseHeaders': {
                            initial() {
                                return true;
                            },
                        },
                        'editor.theme': {
                            initial() {
                                return 'dark';
                            },
                        },
                        'queryPlan.hideQueryPlanResponse': {
                            initial() {
                                return true;
                            },
                        },
                        'request.credentials': {
                            initial() {
                                return 'omit';
                            },
                        },
                        'tracing.hideTracingResponse': {
                            initial() {
                                return true;
                            },
                        },
                    },
                },
            },
        },
        cors: {
            shorthand(enabled) {
                return { enabled, exposedHeaders: [] };
            },
            fields: {
                allowedHeaders: {},
                credentials: {},
                enabled: {},
                exposedHeaders: {},
                maxAge: {},
                methods: {},
                optionsSuccessStatus: {},
                origin: {},
                preflightContinue: {},
            },
        },
        graphql: {
            fields: {
                introspection: {
                    initial() {
                        return process.env.NODE_ENV === 'production' ? false : true;
                    },
                },
            },
        },
        host: {
            initial() {
                var _a, _b;
                return (_b = (_a = process.env.NEXUS_HOST) !== null && _a !== void 0 ? _a : process.env.HOST) !== null && _b !== void 0 ? _b : undefined;
            },
        },
        path: {
            initial() {
                return '/graphql';
            },
            validate(value) {
                if (value.length === 0) {
                    return { reasons: ['Custom GraphQL `path` cannot be empty and must start with a /'] };
                }
                return null;
            },
            fixup(value) {
                const messages = [];
                if (!value.startsWith('/')) {
                    messages.push('Custom GraphQL `path` must start with a "/". Please add it.');
                    value = '/' + value;
                }
                return messages.length ? { value, messages } : null;
            },
        },
        port: {
            initial() {
                return typeof process.env.NEXUS_PORT === 'string'
                    ? parseInt(process.env.NEXUS_PORT, 10)
                    : typeof process.env.PORT === 'string'
                        ? // e.g. Heroku convention https://stackoverflow.com/questions/28706180/setting-the-port-for-node-js-server-on-heroku
                            parseInt(process.env.PORT, 10)
                        : process.env.NODE_ENV === 'production'
                            ? 80
                            : 4000;
            },
        },
        startMessage: {
            initial() {
                return ({ port, host, paths }) => {
                    const url = `http://${Utils.prettifyHost(host)}:${port}${paths.graphql}`;
                    const subscriptionsURL = paths.graphqlSubscriptions
                        ? `http://${Utils.prettifyHost(host)}:${port}${paths.graphqlSubscriptions}`
                        : null;
                    logger_1.log.info('listening', {
                        url,
                        subscriptionsURL,
                    });
                };
            },
        },
    },
});
//# sourceMappingURL=settings.js.map